// ver 2.0 2008/01/06
// common functions 
// calc_massex 
//   calc mass excess
// calc_mass
//   calc mass from mass excess
// get_atomic_number
//   get atomic nuber from the reaction name
double calc_massex(int A, int Z);
double calc_mass(int A, int Z, double massex);
int get_atomic_number(char *reaction, int *a, int *z);
double bethe(double inE, int Zp, double mass, double thickness, int sw, int target);
double slope(double Es, double mass, int Zp, int target);
double get_ex_toi(char *buf);
int get_parameter(char runname[], double *E, int *posi, int *target);

// get the atomic number form the reaction name
int get_atomic_number(char *reaction, int *a, int *z)
{
	FILE *list;
	int readitem, ta[4], tz[4], i;
	char buffer[BUFSIZE], title[BUFSIZE];

	list = fopen("reaction.txt", "r");
	if (list == NULL) {
		return FALSE;
	}

	while (1) {
		if(fgets(buffer,sizeof(buffer),list) == NULL) {
			break;
		}
		readitem = sscanf(buffer, "%s %d %d %d %d %d %d %d %d", 
				title, &ta[1], &tz[1], &ta[0], &tz[0], &ta[2], &tz[2], &ta[3], &tz[3]);

		if (readitem != 9)
			continue;

		// compare n characters from the list
		if (strncmp(title, reaction, strlen(title)) == 0) {
			for (i = 0;i < 4; i++) { 
				a[i] = ta[i];
				z[i] = tz[i];
			}
			fclose(list);
			return TRUE;
		}
	}
	fclose(list);
	printf("%s is not registered reaction.\nplease, define in reaction.txt\n", reaction);
	return FALSE;
}

double calc_massex(int A, int Z)
{
	FILE *data;
	int ta,tz, readitem;
	float massex;
	char buf[BUFSIZE];

	// awm.txt is the list of the mass excess 
	data = fopen("awm.txt", "r");
	if (data == NULL) {
		return 0;
	}

	while (1) {
		if (fgets(buf,sizeof(buf),data) == NULL)
			break;
		readitem = sscanf(buf,"%d %d %f",&ta, &tz, &massex);
		if ((A == ta) && (Z == tz)) {
			fclose(data);
			return (double)massex/(1e3);
		}
	}
	fclose(data);
	return 0;
}

double calc_mass(int A, int Z, double massex)
{
	return ((double)A)*AMU-((double)Z)*ME+massex;
}

// bethe bloch calculation
// Zp projectile charge
// mass projectile mass
// thickness target thickness
// sw = true normal bethe bloch
// sw = false inverse bethe bloch
double bethe(double inE, int Zp, double mass, double thickness, int sw, int target)
{
	double dx, dEdx = 0.; 
	double k1,k2,k3,k4;
	int sign;
	long int i;

	dx = thickness/LOOP;
	// sw == TRUE normal bethe bloch 
	if (sw == TRUE) {
		sign = 1;
	} else {
		sign = -1;
	}

	for(i=0;i<=LOOP;i++){
		k1 = dx*slope(inE,mass,Zp,target);
		k2 = dx*slope(inE+k1/2,mass,Zp,target);
		k3 = dx*slope(inE+k2/2,mass,Zp,target);
		k4 = dx*slope(inE+k3,mass,Zp,target);
		dEdx = (k1 + 2.0e0*k2 + 2.0e0*k3 + k4)/(6.0e0);
		inE = inE + sign*dEdx;
		if (inE <= 0.) break;
	}
					
	return inE; 
}

// bethe bloch calculation for different target
double slope(double Es, double mass, int Zp, int target)
{
	double s,s2,s3, FACvalue[3], returnvalue;

	if (target == SILICON) { 
		FACvalue[0] = FAC*rho_Si*Z_Si*Zp*Zp/A_Si;
		s = - FACvalue[0]*mass/Es*log(4.*ME*Es/I_Si/mass) + 2.*FACvalue[0];
		return s;
	}
	if (target == ALMTRGT) {
		FACvalue[0] = FAC*rho_Al*Z_Al*Zp*Zp/A_Al;
		s = - FACvalue[0]*mass/Es*log(4.*ME*Es/I_Al/mass) + 2.*FACvalue[0];
		return s;
	}
	if (target == CH2TRGT) {
		FACvalue[0] = FAC*rho_C12atCH2*Z_C12*Zp*Zp/A_C12;
		FACvalue[1] = FAC*rho_HatCH2*Z_H*Zp*Zp/A_H;
		s  = - FACvalue[0]*mass/Es*log(4.*ME*Es/I_C12/mass) + 2.*FACvalue[0];
		s2 = - FACvalue[1]*mass/Es*log(4.*ME*Es/I_H/mass) + 2.*FACvalue[1]; 
		returnvalue = rho_CH2*w_CatCH2/rho_C12atCH2*s + rho_CH2*w_HatCH2/rho_HatCH2*s2;
		return returnvalue;
	}
	if (target == C13TRGT) {
		FACvalue[0] = FAC*rho_C13*Z_C13*Zp*Zp/A_C13;
		s = - FACvalue[0]*mass/Es*log(4.*ME*Es/I_C13/mass) + 2.*FACvalue[0];
		return s;
	}
	if (target == MLMNTRGT) {
		FACvalue[0] = FAC*rho_C12atMERANIN*Z_C12*Zp*Zp/A_C12;
		FACvalue[1] = FAC*rho_HatMERANIN*Z_H*Zp*Zp/A_H;
		FACvalue[2] = FAC*rho_NatMERANIN*Z_N*Zp*Zp/A_N;
		s  = -FACvalue[0]*mass/Es*log(4.*ME*Es/I_C12/mass) + 2.*FACvalue[0];
		s2 = -FACvalue[1]*mass/Es*log(4.*ME*Es/I_H/mass) + 2.*FACvalue[1];
		s3 = -FACvalue[2]*mass/Es*log(4.*ME*Es/I_N/mass) + 2.*FACvalue[2];
		returnvalue = rho_MERANIN*w_CatMERANIN/rho_C12atMERANIN*s
			+ rho_MERANIN*w_HatMERANIN/rho_HatMERANIN*s2
			+ rho_MERANIN*w_NatMERANIN/rho_NatMERANIN*s3;

		return returnvalue;
	}
	return 0;
}

// get excited energy from the table of isotopes file
double get_ex_toi(char *title)
{
	FILE *toifile;
	char buffer[BUFSIZE], reactionname[BUFSIZE];
	double Ex, returnE;
	int readitem;

	toifile = fopen("toi.txt", "r");
	if (toifile == NULL) return 0;
	returnE = 0;
	while (1) {
		if (fgets(buffer,sizeof(buffer),toifile) == NULL) 
			break;
		readitem = sscanf(buffer,"%s %lf", reactionname, &Ex);
		if (readitem != 2)
			continue;
		if (strcmp(title, reactionname) == 0) {
			returnE = Ex;
			return returnE;
		}
	}
	printf("ALERT:%s not registered, in toi.txt\n", title);
	fclose(toifile);
	return returnE;
}

// get parameter from rundatalist.txt (position target Energy)
int get_parameter(char runname[], double *E, int *posi, int *target)
{
	FILE *datafile;
	char buffer[BUFSIZE], trunname[BUFSIZE];
	double tE, tFC;
	int readitem, ttarget, tposition;

	datafile = fopen("rundatalist.txt", "r");
	if (datafile == NULL) return FALSE;
	while (1) {
		if (fgets(buffer,sizeof(buffer),datafile) == NULL) 
			break;
		readitem = sscanf(buffer, "%s %d %lf %lf %d",
				trunname, &ttarget, &tE, &tFC, &tposition);
		if (readitem != 5)
			continue;

		if (strncmp(trunname, runname, strlen(runname)) == 0) {
			*target = ttarget;
			*E = tE;
			*posi = tposition;
			fclose(datafile);
			return TRUE;
		}
	}
	fclose(datafile);
	printf("run %s, cannot get parameter, in rundatalist.txt\n", runname);
	return FALSE;
}
